@@ -29,7 +29,7 @@ from samcli.lib.config.samconfig import DEFAULT_ENV, DEFAULT_CONFIG_FILE_NAME
 from samcli.lib.intrinsic_resolver.intrinsics_symbol_table import IntrinsicsSymbolTable
 from samcli.lib.package.ecr_utils import is_ecr_url
 from samcli.lib.package.image_utils import tag_translation, NonLocalImageException, NoImageFoundException
-from samcli.lib.providers.provider import Function, Stack
+from samcli.lib.providers.provider import Function, Stack, get_resource_full_path_by_id, ResourceIdentifier
 from samcli.lib.providers.sam_stack_provider import SamLocalStackProvider
 from samcli.lib.utils.colors import Colored
 from samcli.lib.utils.defaults import get_default_aws_region
@@ -41,6 +41,7 @@ LOG = logging.getLogger(__name__)
 
 
 class GuidedContext:
+    # pylint: disable=too-many-statements
     def __init__(
         self,
         template_file,
@@ -59,6 +60,7 @@ class GuidedContext:
         config_section=None,
         config_env=None,
         config_file=None,
+        disable_rollback=None,
     ):
         self.template_file = template_file
         self.stack_name = stack_name
@@ -89,6 +91,7 @@ class GuidedContext:
         self.end_bold = "\033[0m"
         self.color = Colored()
         self.function_provider = None
+        self.disable_rollback = disable_rollback
 
     @property
     def guided_capabilities(self):
@@ -153,6 +156,9 @@ class GuidedContext:
                 type=FuncParamType(func=_space_separated_list_func_type),
             )
 
+        click.secho("\t#Preserves the state of previously provisioned resources when an operation fails")
+        disable_rollback = confirm(f"\t{self.start_bold}Disable rollback{self.end_bold}", default=self.disable_rollback)
+
         self.prompt_authorization(stacks)
         self.prompt_code_signing_settings(stacks)
 
@@ -194,6 +200,7 @@ class GuidedContext:
         self.config_env = config_env if config_env else default_config_env
         self.config_file = config_file if config_file else default_config_file
         self.confirm_changeset = confirm_changeset
+        self.disable_rollback = disable_rollback
 
     def prompt_authorization(self, stacks: List[Stack]):
         auth_required_per_resource = auth_per_resource(stacks)
@@ -330,7 +337,12 @@ class GuidedContext:
         Dict[str, str]
             A dictionary contains image function logical ID as key, image repository as value.
         """
-        updated_repositories = image_repositories.copy() if image_repositories is not None else {}
+        image_repositories = image_repositories if image_repositories is not None else {}
+        updated_repositories = {}
+        for image_repo_func_id, image_repo_uri in image_repositories.items():
+            repo_full_path = get_resource_full_path_by_id(stacks, ResourceIdentifier(image_repo_func_id))
+            if repo_full_path:
+                updated_repositories[repo_full_path] = image_repo_uri
         self.function_provider = SamFunctionProvider(stacks, ignore_code_extraction_warnings=True)
         manager = CompanionStackManager(stack_name, region, s3_bucket, s3_prefix)
 
@@ -560,6 +572,7 @@ class GuidedContext:
                 confirm_changeset=self.confirm_changeset,
                 capabilities=self._capabilities,
                 signing_profiles=self.signing_profiles,
+                disable_rollback=self.disable_rollback,
             )
 
     @staticmethod
