@@ -13,9 +13,10 @@ from samcli.cli.main import pass_context, common_options, print_cmdline_args
 from samcli.lib.utils.version_checker import check_newer_version
 from samcli.local.common.runtime_template import RUNTIMES, SUPPORTED_DEP_MANAGERS, LAMBDA_IMAGES_RUNTIMES
 from samcli.lib.telemetry.metric import track_command
-from samcli.commands.init.interactive_init_flow import _get_runtime_from_image
+from samcli.commands.init.interactive_init_flow import _get_runtime_from_image, get_architectures, get_sorted_runtimes
 from samcli.commands.local.cli_common.click_mutex import Mutex
 from samcli.lib.utils.packagetype import IMAGE, ZIP
+from samcli.lib.utils.architecture import X86_64, ARM64
 
 LOG = logging.getLogger(__name__)
 
@@ -36,9 +37,11 @@ Common usage:
     \b
     Initializes a new SAM project using project templates without an interactive workflow:
     \b
-    $ sam init --name sam-app --runtime nodejs10.x --dependency-manager npm --app-template hello-world
+    $ sam init --name sam-app --runtime nodejs14.x --dependency-manager npm --app-template hello-world
     \b
-    $ sam init --name sam-app --package-type image --base-image nodejs10.x-base
+    $ sam init --name sam-app --runtime nodejs14.x --architecture arm64
+    \b
+    $ sam init --name sam-app --package-type image --base-image nodejs14.x-base
     \b
     Initializes a new SAM project using custom template in a Git/Mercurial repository
     \b
@@ -132,6 +135,13 @@ def non_interactive_validation(func):
         # check non_interactive_validation for additional validations
     ],
 )
+@click.option(
+    "-a",
+    "--architecture",
+    type=click.Choice([ARM64, X86_64]),
+    help="Architectures your Lambda function will run on",
+    cls=Mutex,
+)
 @click.option(
     "-l",
     "--location",
@@ -142,7 +152,7 @@ def non_interactive_validation(func):
 @click.option(
     "-r",
     "--runtime",
-    type=click.Choice(RUNTIMES),
+    type=click.Choice(get_sorted_runtimes(RUNTIMES)),
     help="Lambda Runtime of your app",
     cls=Mutex,
     not_required=["location", "base_image"],
@@ -211,6 +221,7 @@ def cli(
     location,
     package_type,
     runtime,
+    architecture,
     base_image,
     dependency_manager,
     output_dir,
@@ -231,6 +242,7 @@ def cli(
         PackageType.explicit,
         package_type,
         runtime,
+        architecture,
         base_image,
         dependency_manager,
         output_dir,
@@ -249,6 +261,7 @@ def do_cli(
     pt_explicit,
     package_type,
     runtime,
+    architecture,
     base_image,
     dependency_manager,
     output_dir,
@@ -273,9 +286,9 @@ def do_cli(
     image_bool = name and pt_explicit and base_image
     if location or zip_bool or image_bool:
         # need to turn app_template into a location before we generate
-        templates = InitTemplates(no_interactive)
+        templates = InitTemplates()
         if package_type == IMAGE and image_bool:
-            base_image, runtime = _get_runtime_from_image(base_image)
+            runtime = _get_runtime_from_image(base_image)
             options = templates.init_options(package_type, runtime, base_image, dependency_manager)
             if not app_template:
                 if len(options) == 1:
@@ -291,11 +304,20 @@ def do_cli(
                 package_type, runtime, base_image, dependency_manager, app_template
             )
             no_input = True
-        extra_context = _get_cookiecutter_template_context(name, runtime, extra_context)
+        extra_context = _get_cookiecutter_template_context(name, runtime, architecture, extra_context)
 
         if not output_dir:
             output_dir = "."
-        do_generate(location, package_type, runtime, dependency_manager, output_dir, name, no_input, extra_context)
+        do_generate(
+            location,
+            package_type,
+            runtime,
+            dependency_manager,
+            output_dir,
+            name,
+            no_input,
+            extra_context,
+        )
     else:
         # proceed to interactive state machine, which will call do_generate
         do_interactive(
@@ -303,6 +325,7 @@ def do_cli(
             pt_explicit,
             package_type,
             runtime,
+            architecture,
             base_image,
             dependency_manager,
             output_dir,
@@ -319,13 +342,12 @@ def _deprecate_notification(runtime):
     if runtime in deprecated_runtimes:
         message = (
             f"WARNING: {runtime} is no longer supported by AWS Lambda, please update to a newer supported runtime. "
-            f"SAM CLI will drop support for all deprecated runtimes {deprecated_runtimes} on May 1st. "
             f"See issue: https://github.com/awslabs/aws-sam-cli/issues/1934 for more details."
         )
         LOG.warning(Colored().yellow(message))
 
 
-def _get_cookiecutter_template_context(name, runtime, extra_context):
+def _get_cookiecutter_template_context(name, runtime, architecture, extra_context):
     default_context = {}
     extra_context_dict = {}
 
@@ -335,6 +357,7 @@ def _get_cookiecutter_template_context(name, runtime, extra_context):
     if name is not None:
         default_context["project_name"] = name
 
+    default_context["architectures"] = {"value": get_architectures(architecture)}
     if extra_context is not None:
         try:
             extra_context_dict = json.loads(extra_context)
