@@ -17,6 +17,7 @@ from samcli.lib.telemetry.metric import track_command
 from samcli.cli.cli_config_file import configuration_option, TomlProvider
 from samcli.lib.utils.version_checker import check_newer_version
 from samcli.local.docker.exceptions import ContainerNotStartableException
+from samcli.commands._utils.option_value_processor import process_image_options
 
 LOG = logging.getLogger(__name__)
 
@@ -83,6 +84,7 @@ def cli(
     debug_function,
     container_host,
     container_host_interface,
+    invoke_image,
 ):
     """
     `sam local start-api` command entry point
@@ -112,6 +114,7 @@ def cli(
         debug_function,
         container_host,
         container_host_interface,
+        invoke_image,
     )  # pragma: no cover
 
 
@@ -138,6 +141,7 @@ def do_cli(  # pylint: disable=R0914
     debug_function,
     container_host,
     container_host_interface,
+    invoke_image,
 ):
     """
     Implementation of the ``cli`` method, just separated out for unit testing purposes
@@ -154,6 +158,8 @@ def do_cli(  # pylint: disable=R0914
 
     LOG.debug("local start-api command is called")
 
+    processed_invoke_images = process_image_options(invoke_image)
+
     # Pass all inputs to setup necessary context to invoke function locally.
     # Handler exception raised by the processor for invalid args and print errors
 
@@ -180,6 +186,7 @@ def do_cli(  # pylint: disable=R0914
             shutdown=shutdown,
             container_host=container_host,
             container_host_interface=container_host_interface,
+            invoke_images=processed_invoke_images,
         ) as invoke_context:
 
             service = LocalApiService(lambda_invoke_context=invoke_context, port=port, host=host, static_dir=static_dir)
