@@ -41,7 +41,7 @@ class SamConfig:
         """
         self.filepath = Path(config_dir, filename or DEFAULT_CONFIG_FILE_NAME)
 
-    def get_stage_names(self):
+    def get_stage_configuration_names(self):
         self._read()
         if isinstance(self.document, dict):
             return [stage for stage, value in self.document.items() if isinstance(value, dict)]
