@@ -7,7 +7,6 @@ import platform
 import re
 import shutil
 import tempfile
-import uuid
 import zipfile
 import contextlib
 from contextlib import contextmanager
@@ -15,8 +14,7 @@ from typing import Dict, Optional, cast
 
 import jmespath
 
-from samcli.commands.package import exceptions
-from samcli.commands.package.exceptions import ImageNotFoundError
+from samcli.commands.package.exceptions import ImageNotFoundError, InvalidLocalPathError
 from samcli.lib.package.ecr_utils import is_ecr_url
 from samcli.lib.package.s3_uploader import S3Uploader
 from samcli.lib.utils.hash import dir_checksum
@@ -25,7 +23,7 @@ LOG = logging.getLogger(__name__)
 
 # https://docs.aws.amazon.com/AmazonS3/latest/dev-retired/UsingBucket.html
 _REGION_PATTERN = r"[a-zA-Z0-9-]+"
-_DOT_AMAZONAWS_COM_PATTERN = r"\.amazonaws\.com"
+_DOT_AMAZONAWS_COM_PATTERN = r"\.amazonaws\.com(\.cn)?"
 _S3_URL_REGEXS = [
     # Path-Style (and ipv6 dualstack)
     # - https://s3.Region.amazonaws.com/bucket-name/key name
@@ -110,7 +108,8 @@ def upload_local_image_artifacts(resource_id, resource_dict, property_name, pare
     image_path = jmespath.search(property_name, resource_dict)
 
     if not image_path:
-        raise ImageNotFoundError(property_name=property_name, resource_id=resource_id)
+        message_fmt = "Image not found for {property_name} parameter of {resource_id} resource. \n"
+        raise ImageNotFoundError(property_name=property_name, resource_id=resource_id, message_fmt=message_fmt)
 
     if is_ecr_url(image_path):
         LOG.debug("Property %s of %s is already an ECR URL", property_name, resource_id)
@@ -175,7 +174,7 @@ def upload_local_artifacts(
     if is_local_file(local_path):
         return uploader.upload_with_dedup(local_path)
 
-    raise exceptions.InvalidLocalPathError(resource_id=resource_id, property_name=property_name, local_path=local_path)
+    raise InvalidLocalPathError(resource_id=resource_id, property_name=property_name, local_path=local_path)
 
 
 def resource_not_packageable(resource_dict):
@@ -266,19 +265,6 @@ def make_zip(file_name, source_root):
     return zipfile_name
 
 
-@contextmanager
-def mktempfile():
-    directory = tempfile.gettempdir()
-    filename = os.path.join(directory, uuid.uuid4().hex)
-
-    try:
-        with open(filename, "w+") as handle:
-            yield handle
-    finally:
-        if os.path.exists(filename):
-            os.remove(filename)
-
-
 def copy_to_temp_dir(filepath):
     tmp_dir = tempfile.mkdtemp()
     dst = os.path.join(tmp_dir, os.path.basename(filepath))
