@@ -3,13 +3,14 @@ Class that provides all nested stacks from a given SAM template
 """
 import logging
 import os
-from typing import Optional, Dict, cast, List, Iterator, Tuple
+from typing import Optional, Dict, cast, List, Iterator, Tuple, Union
 from urllib.parse import unquote, urlparse
 
 from samcli.commands._utils.template import get_template_data
 from samcli.lib.providers.exceptions import RemoteStackLocationNotSupported
 from samcli.lib.providers.provider import Stack, get_full_path
 from samcli.lib.providers.sam_base_provider import SamBaseProvider
+from samcli.lib.utils.resources import AWS_CLOUDFORMATION_STACK, AWS_SERVERLESS_APPLICATION
 
 LOG = logging.getLogger(__name__)
 
@@ -107,11 +108,11 @@ class SamLocalStackProvider(SamBaseProvider):
 
             stack: Optional[Stack] = None
             try:
-                if resource_type == SamLocalStackProvider.SERVERLESS_APPLICATION:
+                if resource_type == AWS_SERVERLESS_APPLICATION:
                     stack = SamLocalStackProvider._convert_sam_application_resource(
                         self._template_file, self._stack_path, name, resource_properties
                     )
-                if resource_type == SamLocalStackProvider.CLOUDFORMATION_STACK:
+                if resource_type == AWS_CLOUDFORMATION_STACK:
                     stack = SamLocalStackProvider._convert_cfn_stack_resource(
                         self._template_file, self._stack_path, name, resource_properties
                     )
@@ -152,6 +153,7 @@ class SamLocalStackProvider(SamBaseProvider):
                 resource_properties.get("Parameters", {}), global_parameter_overrides
             ),
             template_dict=get_template_data(location),
+            metadata=resource_properties.get("Metadata", {}),
         )
 
     @staticmethod
@@ -185,6 +187,7 @@ class SamLocalStackProvider(SamBaseProvider):
                 resource_properties.get("Parameters", {}), global_parameter_overrides
             ),
             template_dict=get_template_data(template_url),
+            metadata=resource_properties.get("Metadata", {}),
         )
 
     @staticmethod
@@ -194,6 +197,7 @@ class SamLocalStackProvider(SamBaseProvider):
         name: str = "",
         parameter_overrides: Optional[Dict] = None,
         global_parameter_overrides: Optional[Dict] = None,
+        metadata: Optional[Dict] = None,
     ) -> Tuple[List[Stack], List[str]]:
         """
         Recursively extract stacks from a template file.
@@ -212,6 +216,8 @@ class SamLocalStackProvider(SamBaseProvider):
         global_parameter_overrides: Optional[Dict]
             Optional dictionary of values for SAM template global parameters
             that might want to get substituted within the template and its child templates
+        metadata: Optional[Dict]
+            Optional dictionary of nested stack resource metadata values.
 
         Returns
         -------
@@ -228,6 +234,7 @@ class SamLocalStackProvider(SamBaseProvider):
                 template_file,
                 SamLocalStackProvider.merge_parameter_overrides(parameter_overrides, global_parameter_overrides),
                 template_dict,
+                metadata,
             )
         ]
         remote_stack_full_paths: List[str] = []
@@ -240,10 +247,11 @@ class SamLocalStackProvider(SamBaseProvider):
         for child_stack in current.get_all():
             stacks_in_child, remote_stack_full_paths_in_child = SamLocalStackProvider.get_stacks(
                 child_stack.location,
-                os.path.join(stack_path, name),
+                os.path.join(stack_path, stacks[0].stack_id),
                 child_stack.name,
                 child_stack.parameters,
                 global_parameter_overrides,
+                child_stack.metadata,
             )
             stacks.extend(stacks_in_child)
             remote_stack_full_paths.extend(remote_stack_full_paths_in_child)
@@ -345,3 +353,15 @@ class SamLocalStackProvider(SamBaseProvider):
             stack_file_path = os.path.relpath(os.path.realpath(stack_file_path))
 
         return os.path.normpath(os.path.join(os.path.dirname(stack_file_path), path))
+
+
+def is_local_path(path: Union[Dict, str]) -> bool:
+    return bool(path) and not isinstance(path, dict) and not SamLocalStackProvider.is_remote_url(path)
+
+
+def get_local_path(path: str, parent_path: str) -> str:
+    if path.startswith("file://"):
+        path = unquote(urlparse(path).path)
+    else:
+        path = SamLocalStackProvider.normalize_resource_path(parent_path, path)
+    return path
