@@ -3,6 +3,7 @@ Wraps watchdog to observe file system for any change.
 """
 import logging
 import threading
+import uuid
 from abc import ABC, abstractmethod
 
 from pathlib import Path
@@ -17,6 +18,7 @@ from watchdog.observers import Observer
 from watchdog.events import PatternMatchingEventHandler, FileSystemEvent, FileSystemEventHandler
 from watchdog.observers.api import ObservedWatch, BaseObserver
 
+from samcli.cli.global_config import Singleton
 from samcli.lib.utils.hash import dir_checksum, file_checksum
 from samcli.lib.utils.packagetype import ZIP, IMAGE
 from samcli.local.lambdafn.config import FunctionConfig
@@ -135,6 +137,7 @@ class LambdaFunctionObserver:
         }
 
         self._input_on_change: Callable = on_change
+        self._watch_lock: Lock = threading.Lock()
 
     def _on_zip_change(self, paths: List[str]) -> None:
         """
@@ -169,11 +172,12 @@ class LambdaFunctionObserver:
         package_type: str
             determine if the changed resource is a source code path or an image name
         """
-        changed_functions: List[FunctionConfig] = []
-        for resource in resources:
-            if self._observed_functions[package_type].get(resource, None):
-                changed_functions += self._observed_functions[package_type][resource]
-        self._input_on_change(changed_functions)
+        with self._watch_lock:
+            changed_functions: List[FunctionConfig] = []
+            for resource in resources:
+                if self._observed_functions[package_type].get(resource, None):
+                    changed_functions += self._observed_functions[package_type][resource]
+            self._input_on_change(changed_functions)
 
     def watch(self, function_config: FunctionConfig) -> None:
         """
@@ -189,13 +193,14 @@ class LambdaFunctionObserver:
         ObserverException:
             if not able to observe the input function source path/image
         """
-        if self.get_resources.get(function_config.packagetype, None):
-            resources = self.get_resources[function_config.packagetype](function_config)
-            for resource in resources:
-                functions = self._observed_functions[function_config.packagetype].get(resource, [])
-                functions += [function_config]
-                self._observed_functions[function_config.packagetype][resource] = functions
-                self._observers[function_config.packagetype].watch(resource)
+        with self._watch_lock:
+            if self.get_resources.get(function_config.packagetype, None):
+                resources = self.get_resources[function_config.packagetype](function_config)
+                for resource in resources:
+                    functions = self._observed_functions[function_config.packagetype].get(resource, [])
+                    functions += [function_config]
+                    self._observed_functions[function_config.packagetype][resource] = functions
+                    self._observers[function_config.packagetype].watch(resource)
 
     def unwatch(self, function_config: FunctionConfig) -> None:
         """
@@ -315,7 +320,7 @@ class ImageObserver(ResourceObserver):
         with self._lock:
             self.events.close()
             # wait until the images observer thread got stopped
-            while self._images_observer_thread.is_alive():
+            while self._images_observer_thread and self._images_observer_thread.is_alive():
                 pass
 
 
@@ -327,7 +332,7 @@ class FileObserverException(ObserverException):
 
 class FileObserver(ResourceObserver):
     """
-    A class that will observe some file system paths for any change.
+    A class that will Wrap the Singleton File Observer.
     """
 
     def __init__(self, on_change: Callable) -> None:
@@ -338,7 +343,34 @@ class FileObserver(ResourceObserver):
         on_change:
             Reference to the function that will be called if there is a change in aby of the observed paths
         """
-        self._observed_paths: Dict[str, str] = {}
+        self._group = str(uuid.uuid4())
+        self._single_file_observer = SingletonFileObserver()
+        self._single_file_observer.add_group(self._group, on_change)
+
+    def watch(self, resource: str) -> None:
+        self._single_file_observer.watch(resource, self._group)
+
+    def unwatch(self, resource: str) -> None:
+        self._single_file_observer.unwatch(resource, self._group)
+
+    def start(self):
+        self._single_file_observer.start()
+
+    def stop(self):
+        self._single_file_observer.stop()
+
+
+class SingletonFileObserver(metaclass=Singleton):
+    """
+    A Singleton class that will observe some file system paths for any change for multiple purposes.
+    """
+
+    def __init__(self) -> None:
+        """
+        Initialize the file observer
+        """
+        self._observed_paths_per_group: Dict[str, Dict[str, str]] = {}
+        self._observed_groups_handlers: Dict[str, Callable] = {}
         self._observed_watches: Dict[str, ObservedWatch] = {}
         self._watch_dog_observed_paths: Dict[str, List[str]] = {}
         self._observer: BaseObserver = Observer()
@@ -352,7 +384,7 @@ class FileObserver(ResourceObserver):
 
         self._code_modification_handler.on_modified = self.on_change
         self._code_deletion_handler.on_deleted = self.on_change
-        self._input_on_change: Callable = on_change
+        self._watch_lock = threading.Lock()
         self._lock: Lock = threading.Lock()
 
     def on_change(self, event: FileSystemEvent) -> None:
@@ -366,26 +398,60 @@ class FileObserver(ResourceObserver):
         event: watchdog.events.FileSystemEvent
             Determines that there is a change happened to some file/dir in the observed paths
         """
-        observed_paths = [path for path in self._observed_paths if event.src_path.startswith(path)]
-        if not observed_paths:
-            return
+        with self._watch_lock:
+            LOG.debug("a %s change got detected in path %s", event.event_type, event.src_path)
+            for group, _observed_paths in self._observed_paths_per_group.items():
+                if event.event_type == "deleted":
+                    observed_paths = [
+                        path
+                        for path in _observed_paths
+                        if path == event.src_path
+                        or path in self._watch_dog_observed_paths.get(f"{event.src_path}_False", [])
+                    ]
+                else:
+                    observed_paths = [path for path in _observed_paths if event.src_path.startswith(path)]
 
-        changed_paths = []
-        for path in observed_paths:
-            path_obj = Path(path)
-            # The path got deleted
-            if not path_obj.exists():
-                self._observed_paths.pop(path, None)
-                changed_paths += [path]
-            else:
-                new_checksum = calculate_checksum(path)
-                if new_checksum != self._observed_paths.get(path, None):
-                    changed_paths += [path]
-                    self._observed_paths[path] = new_checksum
-        if changed_paths:
-            self._input_on_change(changed_paths)
+                if not observed_paths:
+                    continue
 
-    def watch(self, resource: str) -> None:
+                LOG.debug("affected paths of this change %s", observed_paths)
+                changed_paths = []
+                for path in observed_paths:
+                    path_obj = Path(path)
+                    # The path got deleted
+                    if not path_obj.exists():
+                        _observed_paths.pop(path, None)
+                        changed_paths += [path]
+                    else:
+                        new_checksum = calculate_checksum(path)
+                        if new_checksum and new_checksum != _observed_paths.get(path, None):
+                            changed_paths += [path]
+                            _observed_paths[path] = new_checksum
+                        else:
+                            LOG.debug("the path %s content does not change", path)
+
+                if changed_paths:
+                    self._observed_groups_handlers[group](changed_paths)
+
+    def add_group(self, group: str, on_change: Callable) -> None:
+        """
+        Add new group to file observer. This enable FileObserver to watch the same path for
+        multiple purposes.
+
+        Parameters
+        ----------
+        group: str
+            unique string define a new group of paths to be watched.
+
+        on_change: Callable
+            The method to be called in case if any path related to this group got changed.
+        """
+        if group in self._observed_paths_per_group:
+            raise Exception(f"The group {group} of paths is already watched")
+        self._observed_paths_per_group[group] = {}
+        self._observed_groups_handlers[group] = on_change
+
+    def watch(self, resource: str, group: str) -> None:
         """
         Start watching the input path. File Observer will keep track of the input path with its hash, to check it later
         if it got really changed or not.
@@ -397,24 +463,34 @@ class FileObserver(ResourceObserver):
         resource: str
             The file/dir path to be observed
 
+        group: str
+            unique string define a new group of paths to be watched.
+
         Raises
         ------
         FileObserverException:
             if the input path is not exist
         """
-        path_obj = Path(resource)
-        if not path_obj.exists():
-            raise FileObserverException("Can not observe non exist path")
+        with self._watch_lock:
+            path_obj = Path(resource)
+            if not path_obj.exists():
+                raise FileObserverException("Can not observe non exist path")
 
-        self._observed_paths[resource] = calculate_checksum(resource)
+            _observed_paths = self._observed_paths_per_group[group]
+            _check_sum = calculate_checksum(resource)
+            if not _check_sum:
+                raise Exception(f"Failed to calculate the hash of resource {resource}")
+            _observed_paths[resource] = _check_sum
 
-        # recursively watch the input path, and all child path for any modification
-        self._watch_path(resource, resource, self._code_modification_handler, True)
+            LOG.debug("watch resource %s", resource)
+            # recursively watch the input path, and all child path for any modification
+            self._watch_path(resource, resource, self._code_modification_handler, True)
 
-        # watch only the direct parent path child directories for any deletion
-        # Parent directory watching is needed, as if the input path got deleted,
-        # watchdog will not send an event for it
-        self._watch_path(str(path_obj.parent), resource, self._code_deletion_handler, False)
+            LOG.debug("watch resource %s's parent %s", resource, str(path_obj.parent))
+            # watch only the direct parent path child directories for any deletion
+            # Parent directory watching is needed, as if the input path got deleted,
+            # watchdog will not send an event for it
+            self._watch_path(str(path_obj.parent), resource, self._code_deletion_handler, False)
 
     def _watch_path(
         self, watch_dog_path: str, original_path: str, watcher_handler: FileSystemEventHandler, recursive: bool
@@ -435,17 +511,23 @@ class FileObserver(ResourceObserver):
             determines if we need to watch the path, and all children paths recursively, or just the direct children
             paths
         """
+
+        # Allow watching the same path in 2 Modes recursivly, and non-recusrsivly.
+        # here, we need to only watch the input path in a specific recursive mode
+        original_watch_dog_path = watch_dog_path
+        watch_dog_path = f"{watch_dog_path}_{recursive}"
         child_paths = self._watch_dog_observed_paths.get(watch_dog_path, [])
         first_time = not bool(child_paths)
         if original_path not in child_paths:
             child_paths += [original_path]
         self._watch_dog_observed_paths[watch_dog_path] = child_paths
         if first_time:
+            LOG.debug("Create Observer for resource %s with recursive %s", original_watch_dog_path, recursive)
             self._observed_watches[watch_dog_path] = self._observer.schedule(
-                watcher_handler, watch_dog_path, recursive=recursive
+                watcher_handler, original_watch_dog_path, recursive=recursive
             )
 
-    def unwatch(self, resource: str) -> None:
+    def unwatch(self, resource: str, group: str) -> None:
         """
         Remove the input path form the observed paths, and stop watching this path.
 
@@ -453,16 +535,20 @@ class FileObserver(ResourceObserver):
         ----------
         resource: str
             The file/dir path to be unobserved
+        group: str
+            unique string define a new group of paths to be watched.
         """
         path_obj = Path(resource)
 
+        LOG.debug("unwatch resource %s", resource)
         # unwatch input path
-        self._unwatch_path(resource, resource)
+        self._unwatch_path(resource, resource, group, True)
 
+        LOG.debug("unwatch resource %s's parent %s", resource, str(path_obj.parent))
         # unwatch parent path
-        self._unwatch_path(str(path_obj.parent), resource)
+        self._unwatch_path(str(path_obj.parent), resource, group, False)
 
-    def _unwatch_path(self, watch_dog_path: str, original_path: str) -> None:
+    def _unwatch_path(self, watch_dog_path: str, original_path: str, group: str, recursive: bool) -> None:
         """
         update the observed paths data structure, and call watch dog observer to unobserve the input watch dog path
         if it is not observed before
@@ -473,14 +559,26 @@ class FileObserver(ResourceObserver):
             The file/dir path to be unobserved by watch dog
         original_path: str
             The original input file/dir path to be unobserved
+        group: str
+            unique string define a new group of paths to be watched.
+        recursive: bool
+            determines if we need to watch the path, and all children paths recursively, or just the direct children
+            paths
         """
+
+        # Allow watching the same path in 2 Modes recursivly, and non-recusrsivly.
+        # here, we need to only stop watching the input path in a specific recursive mode
+        original_watch_dog_path = watch_dog_path
+        watch_dog_path = f"{watch_dog_path}_{recursive}"
+        _observed_paths = self._observed_paths_per_group[group]
         child_paths = self._watch_dog_observed_paths.get(watch_dog_path, [])
         if original_path in child_paths:
             child_paths.remove(original_path)
-            self._observed_paths.pop(original_path, None)
+            _observed_paths.pop(original_path, None)
         if not child_paths:
             self._watch_dog_observed_paths.pop(watch_dog_path, None)
             if self._observed_watches.get(watch_dog_path, None):
+                LOG.debug("Unschedule Observer for resource %s with recursive %s", original_watch_dog_path, recursive)
                 self._observer.unschedule(self._observed_watches[watch_dog_path])
                 self._observed_watches.pop(watch_dog_path, None)
 
@@ -501,10 +599,13 @@ class FileObserver(ResourceObserver):
                 self._observer.stop()
 
 
-def calculate_checksum(path: str) -> str:
-    path_obj = Path(path)
-    if path_obj.is_file():
-        checksum = file_checksum(path)
-    else:
-        checksum = dir_checksum(path)
-    return checksum
+def calculate_checksum(path: str) -> Optional[str]:
+    try:
+        path_obj = Path(path)
+        if path_obj.is_file():
+            checksum = file_checksum(path)
+        else:
+            checksum = dir_checksum(path)
+        return checksum
+    except Exception:
+        return None
