@@ -5,6 +5,7 @@ import logging
 import io
 
 from flask import Flask, request
+from werkzeug.routing import BaseConverter
 
 from samcli.lib.utils.stream_writer import StreamWriter
 from samcli.local.services.base_local_service import BaseLocalService, LambdaOutputParser
@@ -14,6 +15,16 @@ from .lambda_error_responses import LambdaErrorResponses
 LOG = logging.getLogger(__name__)
 
 
+class FunctionNamePathConverter(BaseConverter):
+    regex = ".+"
+
+    def to_python(self, value):
+        return value
+
+    def to_url(self, value):
+        return value
+
+
 class LocalLambdaInvokeService(BaseLocalService):
     def __init__(self, lambda_runner, port, host, stderr=None):
         """
@@ -40,7 +51,10 @@ class LocalLambdaInvokeService(BaseLocalService):
         """
         self._app = Flask(__name__)
 
-        path = "/2015-03-31/functions/<function_name>/invocations"
+        # add converter to support nested stack function path
+        self._app.url_map.converters["function_path"] = FunctionNamePathConverter
+
+        path = "/2015-03-31/functions/<function_path:function_name>/invocations"
         self._app.add_url_rule(
             path,
             endpoint=path,
